
subroutine CalcEnergy(Pos, M, L, rc, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc
    real(8), intent(out) :: PEnergy
    real(8), parameter :: k = 3000., r0 = 1.
    real(8), dimension(Dim) :: rij, Fij, Posi 
    real(8) :: z, iz, iz3, iz9, zz, izz, izz3, izz9, d2, id2, id6, id12 
    real(8) :: rc2, Shift, d
    integer :: i, j 
    PEnergy = 0. 
    Shift = -(rc**(-12) - rc**(-6))
    rc2 = rc * rc 
    do i = 0, NAtom - 1 
      Posi = Pos(i,:)
	z = Pos(i,2)
	iz = 1./z
	iz3 = iz * iz * iz
	iz9 = iz3 * iz3 * iz3
	zz = L - z
	izz = 1./zz
	izz3 = izz * izz * izz
	izz9 = izz3 * izz3 * izz3
 	if (mod(i,M) == 0) then
		PEnergy = PEnergy + 2.066 * (iz9 + izz9) + 15.499 * (-iz3 - izz3)
	else
		PEnergy = PEnergy + 2.066 * (iz9 + izz9)
	end if 
      	do j = i + 1, NAtom - 1 
            	rij = Pos(j,:) - Posi 
            	rij(:2) = rij(:2) - L * dnint(rij(:2) / L)  
            	d2 = sum(rij * rij) 
			if (j == i + 1 .and. mod(j, M) > 0) then
				d = sqrt(d2) 
				PEnergy = PEnergy + (k/2.)*((d-r0)**2)
			else if (d2 <= rc2 .and. mod(i,M) == 0) then
		      	!PP and PH LJ interaction
                		id2 = 1. / d2        
                		id6 = id2 * id2 * id2 
                		id12 = id6 * id6       
                		PEnergy = PEnergy + (id12 - id6) + Shift
			else if (d2 <= rc2 .and. mod(j,M) == 0) then
		      	!PP and PH LJ interaction
                		id2 = 1. / d2        
                		id6 = id2 * id2 * id2 
                		id12 = id6 * id6       
                		PEnergy = PEnergy + (id12 - id6) + Shift
			else if (d2 <= rc2 .and. mod(i,M) > 0 .and. mod(j,M) > 0) then
		     		!HH LJ interaction
                		id2 = 1. / d2        
                		id6 = id2 * id2 * id2 
                		id12 = id6 * id6       
                		PEnergy = PEnergy + 4. * (id12 - id6) + 4. * Shift
             	else 
                		cycle
		 	end if
        	enddo 
    enddo 
end subroutine



subroutine CalcEnergyForces(Pos, M, L, rc, PEnergy, Forces, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
!f2py intent(in, out) :: Forces
    real(8), parameter :: k = 3000., r0 = 1.
    real(8), dimension(Dim) :: rij, Fij, Posi 
    real(8) :: z, iz, iz3, iz4, iz9, iz10, zz, izz, izz3, izz4, izz9, izz10, d2, id2, id6, id12
    real(8) :: rc2, Shift, d 
    integer :: i, j 
    PEnergy = 0. 
    Forces = 0. 
    Shift = -(rc**(-12) - rc**(-6)) 
    rc2 = rc * rc 
    do i = 0, NAtom - 1 
        Posi = Pos(i,:)
	  z = Pos(i,2)
	  iz = 1./z
	  iz3 = iz * iz * iz
	  iz4 = iz3 * iz
	  iz9 = iz3 * iz3 * iz3
	  iz10 = iz9 * iz
	  zz = L - z
	  izz = 1./zz
	  izz3 = izz * izz * izz
	  izz4 = izz3 * izz
	  izz9 = izz3 * izz3 * izz3
	  izz10 = izz9 * izz
 	  if (mod(i,M) == 0) then
		 PEnergy = PEnergy + 2.066 * (iz9 + izz9) + 15.499 * (-iz3 - izz3)
		 Forces(i,2) = Forces(i,2) + 18.598 * (iz10 - izz10) + 46.497 * (-iz4 + izz4)
	  else
		 PEnergy = PEnergy + 2.066 * (iz9 + izz9)
		 Forces(i,2) = Forces(i,2) + 18.598 * (iz10 - izz10)
	  end if 
        do j = i + 1, NAtom - 1 
            rij = Pos(j,:) - Posi 
            rij(:2) = rij(:2) - L * dnint(rij(:2) / L)  
            d2 = sum(rij * rij)
		if (j == i + 1 .and. mod(j, M) > 0) then
		     d = sqrt(d2) 
		     PEnergy = PEnergy + (k/2.)*((d-r0)**2)
		     Fij = rij*k*(1-r0/d)
		     Forces(i,:) = Forces(i,:) + Fij 
                 Forces(j,:) = Forces(j,:) - Fij  
		else if (d2 <= rc2 .and. mod(i,M) == 0) then
		     !PP and PH LJ interaction
                 id2 = 1. / d2        
                 id6 = id2 * id2 * id2 
                 id12 = id6 * id6       
                 PEnergy = PEnergy + (id12 - id6) + Shift
		     Fij = rij * ((-12. * id12 + 6. * id6) * id2) 
                 Forces(i,:) = Forces(i,:) + Fij 
                 Forces(j,:) = Forces(j,:) - Fij 
		else if (d2 <= rc2 .and. mod(j,M) == 0) then
		     !PP and PH LJ interaction
                 id2 = 1. / d2        
                 id6 = id2 * id2 * id2 
                 id12 = id6 * id6       
                 PEnergy = PEnergy + (id12 - id6) + Shift
		     Fij = rij * ((-12. * id12 + 6. * id6) * id2) 
                 Forces(i,:) = Forces(i,:) + Fij 
                 Forces(j,:) = Forces(j,:) - Fij 
		 else if (d2 <= rc2 .and. mod(i,M) > 0 .and. mod(j,M) > 0) then
		     !HH LJ interaction
                 id2 = 1. / d2        
                 id6 = id2 * id2 * id2 
                 id12 = id6 * id6       
                 PEnergy = PEnergy + 4. * (id12 - id6) + 4. * Shift
		     Fij = rij * 4. * ((-12. * id12 + 6. * id6) * id2) 
                 Forces(i,:) = Forces(i,:) + Fij 
                 Forces(j,:) = Forces(j,:) - Fij 
             else 
                cycle
		 end if
        enddo 
    enddo 
end subroutine


subroutine EquilibCalcEnergy(Pos, M, L, rc, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc
    real(8), intent(out) :: PEnergy
    real(8), parameter :: k = 3000., r0 = 1.
    real(8), dimension(Dim) :: rij, Fij, Posi 
    real(8) :: d2, id2, id6, id12 
    real(8) :: rc2, Shift, d
    integer :: i, j 
    PEnergy = 0. 
    Shift = -(rc**(-12) - rc**(-6))
    rc2 = rc * rc 
    do i = 0, NAtom - 1 
	Posi = Pos(i,:)
    	do j = i + 1, NAtom - 1 
      	rij = Pos(j,:) - Posi 
            rij(:2) = rij(:2) - L * dnint(rij(:2) / L)  
            d2 = sum(rij * rij) 
		if (j == i + 1 .and. mod(j, M) > 0) then
			d = sqrt(d2) 
			PEnergy = PEnergy + (k/2.)*((d-r0)**2)
		else if (d2 <= rc2 .and. mod(i,M) == 0) then
		      !PP and PH LJ interaction
                	id2 = 1. / d2        
                	id6 = id2 * id2 * id2 
                	id12 = id6 * id6       
                	PEnergy = PEnergy + (id12 - id6) + Shift
		else if (d2 <= rc2 .and. mod(j,M) == 0) then
		      !PP and PH LJ interaction
                	id2 = 1. / d2        
                	id6 = id2 * id2 * id2 
                	id12 = id6 * id6       
                	PEnergy = PEnergy + (id12 - id6) + Shift
		else if (d2 <= rc2 .and. mod(i,M) > 0 .and. mod(j,M) > 0) then
		     	!HH LJ interaction
                	id2 = 1. / d2        
                	id6 = id2 * id2 * id2 
                	id12 = id6 * id6       
                	PEnergy = PEnergy + 4. * (id12 - id6) + 4. * Shift
             else 
                	cycle
		 end if
        enddo 
    enddo 
end subroutine



subroutine EquilibCalcEnergyForces(Pos, M, L, rc, PEnergy, Forces, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
!f2py intent(in, out) :: Forces
    real(8), parameter :: k = 3000., r0 = 1.
    real(8), dimension(Dim) :: rij, Fij, Posi 
    real(8) :: d2, id2, id6, id12
    real(8) :: rc2, Shift, d 
    integer :: i, j 
    PEnergy = 0. 
    Forces = 0. 
    Shift = -(rc**(-12) - rc**(-6)) 
    rc2 = rc * rc 
    do i = 0, NAtom - 1 
        Posi = Pos(i,:)
        do j = i + 1, NAtom - 1 
            rij = Pos(j,:) - Posi 
            rij(:2) = rij(:2) - L * dnint(rij(:2) / L)  
            d2 = sum(rij * rij)
		if (j == i + 1 .and. mod(j, M) > 0) then
		     d = sqrt(d2) 
		     PEnergy = PEnergy + (k/2.)*((d-r0)**2)
		     Fij = rij*k*(1-r0/d)
		     Forces(i,:) = Forces(i,:) + Fij 
                 Forces(j,:) = Forces(j,:) - Fij  
		else if (d2 <= rc2 .and. mod(i,M) == 0) then
		     !PP and PH LJ interaction
                 id2 = 1. / d2        
                 id6 = id2 * id2 * id2 
                 id12 = id6 * id6       
                 PEnergy = PEnergy + (id12 - id6) + Shift
		     Fij = rij * ((-12. * id12 + 6. * id6) * id2) 
                 Forces(i,:) = Forces(i,:) + Fij 
                 Forces(j,:) = Forces(j,:) - Fij 
		else if (d2 <= rc2 .and. mod(j,M) == 0) then
		     !PP and PH LJ interaction
                 id2 = 1. / d2        
                 id6 = id2 * id2 * id2 
                 id12 = id6 * id6       
                 PEnergy = PEnergy + (id12 - id6) + Shift
		     Fij = rij * ((-12. * id12 + 6. * id6) * id2) 
                 Forces(i,:) = Forces(i,:) + Fij 
                 Forces(j,:) = Forces(j,:) - Fij 
		 else if (d2 <= rc2 .and. mod(i,M) > 0 .and. mod(j,M) > 0) then
		     !HH LJ interaction
                 id2 = 1. / d2        
                 id6 = id2 * id2 * id2 
                 id12 = id6 * id6       
                 PEnergy = PEnergy + 4. * (id12 - id6) + 4. * Shift
		     Fij = rij * 4.*((-12. * id12 + 6. * id6) * id2) 
                 Forces(i,:) = Forces(i,:) + Fij 
                 Forces(j,:) = Forces(j,:) - Fij 
             else 
                cycle
		 end if
        enddo 
    enddo 
end subroutine
 
    
subroutine VVIntegrate(Pos, Vel, Accel, M, L, rc, dt, KEnergy, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in) :: L, rc, dt
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos, M, L, rc, PEnergy, Accel, Dim, NAtom)
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine

subroutine EquilibVVIntegrate(Pos, Vel, Accel, M, L, rc, dt, KEnergy, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in) :: L, rc, dt
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: EquilibCalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    Vel = Vel + 0.5 * dt * Accel
    call EquilibCalcEnergyForces(Pos, M, L, rc, PEnergy, Accel, Dim, NAtom)
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine
